#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_rename.h"

static int le_rename;

ZEND_BEGIN_ARG_INFO_EX(arginfo_function_rename, 0, 0, 2)
	ZEND_ARG_INFO(0, orig_func)
	ZEND_ARG_INFO(0, new_func)
ZEND_END_ARG_INFO()

/* {{{ rename_functions[]
 *
 * Every user visible function must have an entry in rename_functions[].
 */
const zend_function_entry rename_functions[] = {
	PHP_FE(function_rename, arginfo_function_rename)
	PHP_FE_END
};
/* }}} */

/* {{{ rename_module_entry
 */
zend_module_entry rename_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"rename",
	rename_functions,
	PHP_MINIT(rename),
	PHP_MSHUTDOWN(rename),
	NULL,
	NULL,
	PHP_MINFO(rename),
#if ZEND_MODULE_API_NO >= 20010901
	"0.1",
#endif
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_RENAME
ZEND_GET_MODULE(rename)
#endif

/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION(rename)
{
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MSHUTDOWN_FUNCTION
 */
PHP_MSHUTDOWN_FUNCTION(rename)
{
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_RINIT_FUNCTION
*/
PHP_RINIT_FUNCTION(comuto)
{
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_RSHUTDOWN_FUNCTION
*/
PHP_RSHUTDOWN_FUNCTION(comuto)
{
	return SUCCESS;
}

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(rename)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "rename support", "enabled");
	php_info_print_table_end();
}
/* }}} */

/* {{{ proto string function_rename(string orig_func, string new_func)
*/
PHP_FUNCTION(function_rename)
{
	char *orig_func, *new_func;
	int orig_func_len, new_func_len;
	zend_function *func;

	if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss", &orig_func, &orig_func_len, &new_func, &new_func_len) == FAILURE) {
		return;
	}

	if(zend_hash_find(EG(function_table), orig_func, orig_func_len + 1, (void **) &func) == FAILURE) {
		zend_error(E_WARNING, "cannot find %s()", orig_func);
		RETURN_FALSE;
	}

	if(zend_hash_exists(EG(function_table), new_func, new_func_len + 1)) {
		zend_error(E_WARNING, "%s() already exists", new_func);
		RETURN_FALSE;
	}

	if(zend_hash_add(EG(function_table), new_func, new_func_len + 1, func, sizeof(zend_function), NULL) == FAILURE) {
		zend_error(E_WARNING, "cannot rename %s()", orig_func);
		RETURN_FALSE;
	}

	if(zend_hash_del(EG(function_table), orig_func, orig_func_len + 1) == FAILURE) {
		zend_error(E_WARNING, "cannot rename %s()", orig_func);
		RETURN_FALSE;
	}

	RETURN_TRUE;
}